#ifndef __USB_H__
#define __USB_H__

#include <unistd.h>
#include <errno.h>

#include <linux/usb_ch9.h>
#include <linux/usbdevice_fs.h>

struct opninfo;

struct urbs {
	struct urbs* next;
	struct usbdevfs_urb* urb;
	struct opninfo* opn;
};

#define USB_REQUEST_QUEUED	(-2)

struct usbdev_fn {
	int (*USBControl)(void*, struct usbdevfs_ctrltransfer*);
	int (*USBXfer)(void*, struct urbs* urbs, unsigned int type);
	int (*USBClaimInterface)(void*, int interface);
	int (*USBReleaseInterface)(void*, int interface);
	int (*USBDiscardUrb)(void*, struct usbdevfs_urb* urb);
	void (*USBForkCleanup)(void*);
};

size_t get_registered_usbdevs(u_int32_t* devlist, size_t maxlen);
int register_usbdev(void* cookie, struct usbdev_fn* fn, int busid, int devid);
void* usbdev_find(int busid, int devid);
void usbdev_puturb(struct urbs* urbs);
const char* vmxGetConfig(void);

static inline uid_t
getSuperUser(void)
{
	return geteuid();
}

static inline void
restoreSuperUser(uid_t euid)
{
	int savedErrno = errno;
	seteuid(euid);
	errno = savedErrno;
}

static inline uid_t
becomeSuperUser(void)
{
	uid_t su;

	su = getSuperUser();
	if (su) {
		restoreSuperUser(0);
	}
	return su;
}

void restoreNormalUser(uid_t euid);
uid_t becomeNormalUser(void);

void usb_log(const char* msg, ...);
void usb_warn(const char* msg, ...);

#define USB_LANGID_cs_CZ	0x0405
#define USB_LANGID_en_US	0x0409

#define USB_DT_CS_INTERFACE	0x24

#endif

