#ifndef __USBCONTROL_H__
#define __USBCONTROL_H__

#include <sys/types.h>

unsigned char* stuffString(unsigned char* buffer, const char* str);
unsigned char* stuffBoolean(unsigned char* buffer, unsigned int writeable, const char* name, int value);
unsigned char* stuffInteger(unsigned char* buffer, unsigned int writeable, const char* name, unsigned int value);
unsigned char* stuffFileName(unsigned char* buffer, unsigned int writeable, const char* name, const char* value);

int keycmp(const unsigned char* key, const char* val);
const unsigned char* fetchChange(const unsigned char* ptr, const unsigned char* end, const unsigned char** key, char* type, const unsigned char** val);

int fetchBoolean(int type, const unsigned char* val);
char* fetchString(int type, const unsigned char* val);
void retrieveBoolean(int* v, int type, const unsigned char* val);
int retrieveInteger(unsigned int* v, int type, const unsigned char* val);
void retrieveString(char** v, int type, const unsigned char* val);

static inline void setLE16(unsigned char* b, unsigned int val) {
	*(u_int16_t*)b = val;
}

static inline void setLE32(unsigned char* b, unsigned int val) {
	*(u_int32_t*)b = val;
}

static inline u_int32_t getLE32(const unsigned char* b) {
	return *(const u_int32_t*)b;
}

static inline void setBE16(unsigned char* b, unsigned int val) {
        b[0] = val >> 8;
        b[1] = val;
}

static inline void setBE32(unsigned char* b, unsigned int val) {
        b[0] = val >> 24;
        b[1] = val >> 16;
        b[2] = val >>  8;
        b[3] = val;
}

static inline unsigned int getBE32(const unsigned char* b) {
        return (b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3];
}

static inline unsigned int getBE16(const unsigned char* b) {
        return (b[0] << 8) | b[1];
}

int usbcontrolStartWatchingFileDescriptor(int fd,
					  void* cookie,
					  int (*fdready)(void* cookie, int fd, int revents),
					  int events);
int usbcontrolStopWatchingFileDescriptor(int fd);

void usbcontrolInit(void);
void usbcontrolForkCleanup(void);

#endif	/* __USBCONTROL_H__ */

