#ifndef __USBDEV_H__
#define __USBDEV_H__

#include "usb.h"

#define list_entry(ptr, type, member) ({                      \
        const typeof( ((type *)0)->member ) *__mptr = (ptr);  \
        (type *)( (char *)__mptr - offsetof(type,member) );})

int usb_copy_desc(int dstlen, void* dst, int srclen, const void* src);
int usb_set_string(void* buffer, const char* name);

struct dynarray {
	unsigned int len;
	unsigned int allocated;
	void** ptr;
};

struct usbendpoint_fns {
	void (*forkCleanup)(void* cookie);
};

struct usbinterface_fns {
	int (*intfDeviceControl)(void* cookie, struct usbdevfs_ctrltransfer* ct);
	int (*intfGetDescriptor)(void* cookie, struct usb_interface_descriptor* uid);
	void (*intfSelected)(void* cookie);
	int (*intfGetFunctionalDescriptor)(void* cookie, unsigned char* buffer, unsigned int idx);
	void (*forkCleanup)(void* cookie);
};

struct usbconfig_fns {
	void (*forkCleanup)(void* cookie);
};

struct usbdevice_fns {
	int (*getdesc)(void* cookie, int type, int idx, int language, int size, void* data);
	void (*forkCleanup)(void* cookie);
};

struct usblogdev {
	u_int32_t seq;
	int (*getName)(struct usblogdev* ul, unsigned char* buffer);
	int (*getSettings)(struct usblogdev* ul, unsigned char* buffer);
	int (*setSettings)(struct usblogdev* ul, const unsigned char* inp, size_t inplen, unsigned char* buffer);
	void (*forkCleanup)(struct usblogdev* ul);
};

#define UE_STALL_NONE		0x00
#define UE_STALL_ONCE		0x01
#define UE_STALL_FOREVER	0x03

struct queuedXfer {
	struct queuedXfer* next;
	int (*xfer)(struct queuedXfer*, size_t len, void* data);
	int (*abort)(struct queuedXfer*);
};

struct usbendpoint {
	unsigned char bEndpointAddress;
	unsigned int bmAttributes;
	unsigned int wMaxPacketSize;
	unsigned char bInterval;
	void* endpointCookie;
	pthread_mutex_t urbsLock;
	struct urbs* urbs;
	int maxQueueLength;
	int queueLength;
	struct queuedXfer* queuedInputXfers;
	struct queuedXfer** nextQueuedInputXfer;
	int (*inputXfer)(void* cookie, size_t len, void* data);
	int (*outputXfer)(void* cookie, size_t len, const void* data);
	unsigned int stalled : 2;
	struct usbendpoint_fns* fns;
};

struct usbinterface {
	struct dynarray endpoints;
	unsigned int claimed:1;
	struct usbinterface_fns *fns;
	void*			intfCookie;
	struct usbconfig*	config;
	struct usbinterface*	parent;
	struct usbinterface*	nextAlternative;
	u_int8_t		bInterfaceNumber;
	u_int8_t		bAlternateSetting;
};

struct usbconfig {
	struct dynarray interfaces;
	struct dynarray logdevs;
	void* configCookie;
	struct dynarray iendpoints, oendpoints;
	struct usbconfig_fns* fns;
};

struct usbdevice {
	struct dynarray configs;
	int activeConfig;
	struct usbdevice_fns* fns;
	void* devcookie;
	struct dynarray strings;
	pthread_mutex_t stringsLock;
	struct usb_device_descriptor desc;
};

#if 0
void dynarray_init(struct dynarray* p);
void dynarray_done(struct dynarray* p);
int dynarray_append(struct dynarray* p, void* newelem);
#endif
void* dynarray_get(const struct dynarray* p, unsigned int index);

void usbendpointInit(struct usbendpoint* ue);
void usbendpointDestroy(struct usbendpoint* ue);

int
usbendpointIACK(struct usbendpoint* ue, int (*inputXfer)(void* cookie, size_t len, void* data), int stall);
int
usbendpointQueueIACK(struct usbendpoint* ue, struct queuedXfer* xfer);

static inline int
usbendpointIStall(struct usbendpoint* ue)
{
	return usbendpointIACK(ue, NULL, UE_STALL_FOREVER);
}

static inline int 
usbendpointINAK(struct usbendpoint* ue)
{
	return usbendpointIACK(ue, NULL, 0);
}

int usbendpointOACK(struct usbendpoint* ue);
int usbendpointOStall(struct usbendpoint* ue);
#if 0
int usbendpointGetDescriptor(struct usbendpoint* ue, void* buffer);
int usbendpointDeviceControl(struct usbendpoint* ue, struct usbdevfs_ctrltransfer* ct);
#endif

void usbinterfaceInit(struct usbinterface* ui);
void usbinterfaceDestroy(struct usbinterface* ui);
int usbinterfaceAddEndpoint(struct usbinterface* ui, struct usbendpoint* ue);
int usbinterfaceRemoveEndpoint(struct usbinterface* ui, struct usbendpoint* ue);
int usbinterfaceAddAlternative(struct usbinterface* parent, struct usbinterface* alternative);
#if 0
struct usbendpoint* usbinterfaceGetEndpoint(struct usbinterface* ui, unsigned int epn);
int usbinterfaceSetInterface(struct usbinterface* ui, int intf, int alts);
int usbinterfaceDeviceControl(struct usbinterface* ui, struct usbdevfs_ctrltransfer* ct);
int usbinterfaceGetDescriptor(struct usbinterface* ui, unsigned int idx, void* buffer);
#endif
int usbinterfaceIsClaimed(const struct usbinterface* ui);

void usbconfigInit(struct usbconfig* uc);
#if 0
struct usbinterface* usbconfigGetInterface(struct usbconfig* uc, int intf);
#endif
int usbconfigAddInterface(struct usbconfig* uc, struct usbinterface* ui);
struct usblogdev* usbconfigGetLogicalDevice(const struct usbconfig* uc, int logdev);
int usbconfigAddLogicalDevice(struct usbconfig* uc, struct usblogdev* ul);
int usbconfigRemoveLogicalDevice(struct usbconfig* uc, struct usblogdev* ul);
int usbconfigAssignEndpoints(struct usbconfig* uc);
int usbconfigRemoveInterface(struct usbconfig* uc, struct usbinterface* ui);
#if 0
struct usbendpoint* usbconfigGetEndpoint(struct usbconfig* uc, int ep);
int init_usb_config_descriptor(struct usbconfig* uc, unsigned int idx, void* buffer);
int usbconfigGetConfigDescriptor(struct usbconfig* uc, unsigned int idx, size_t size, void* data);
#endif
void usbdeviceInit(struct usbdevice* ud);
#if 0
struct usbconfig* usbdeviceGetConfig(struct usbdevice* ud, int cfg);
#endif
struct usbconfig* usbdeviceCurrentConfig(const struct usbdevice* ud);
int usbdeviceAddConfig(struct usbdevice* ud, struct usbconfig* uc);
struct usbinterface* usbdeviceGetInterface(struct usbdevice* ud, int intf);
int usbdeviceAddStringW(struct usbdevice* ud, const u_int16_t* msg, int unique);
int usbdeviceAddString(struct usbdevice* ud, const char* msg, int unique);
int usbdeviceModifyStringW(struct usbdevice* ud, int id, const u_int16_t* msg);
int usbdeviceModifyString(struct usbdevice* ud, int id, const char* msg);
int usbdeviceRemoveString(struct usbdevice* ud, int id);

#if 0
struct usbendpoint* usbdeviceGetEndpoint(struct usbdevice* ud, int ep);
int usbdeviceGetDescriptor(struct usbdevice* ud, unsigned int type, unsigned int idx, unsigned int language, size_t size, void* data);
int usbdeviceDeviceControl(struct usbdevice* ud, struct usbdevfs_ctrltransfer* ct);
int usbdeviceControl(void* cookie, struct usbdevfs_ctrltransfer* ct);
int usbdeviceXfer(void* cookie, struct usbdevfs_bulktransfer* bt, unsigned int type);
int usbdeviceClaimInterface(void* cookie, int num);
int usbdeviceReleaseInterface(void* cookie, int num);
void usbdeviceAssignEndpoints(struct usbdevice* ud);
#endif
int usbdeviceRegister(struct usbdevice* ud, int busid, int devid);
struct usbdevice* usbdeviceFind(int busid, int devid);
int usbdeviceIsClaimed(const struct usbdevice *ud);

int usbdeviceControl(void* cookie, struct usbdevfs_ctrltransfer* ct);

static inline int 
usbdeviceGetDescriptor(struct usbdevice* ud,	// IN: device
		       unsigned int desctype,	// IN: descriptor type
		       unsigned int number,	// IN: number
		       unsigned int language,	// IN: language or 0
		       size_t len,		// IN: length,
		       void* desc)		// OUT: descriptor
{
	struct usbdevfs_ctrltransfer ct;
	
	ct.bRequestType = USB_DIR_IN | USB_TYPE_STANDARD | USB_RECIP_DEVICE;
	ct.bRequest = USB_REQ_GET_DESCRIPTOR;
	ct.wValue = (desctype << 8) | number;
	ct.wIndex = language;
	ct.wLength = len;
	ct.data = desc;
	return usbdeviceControl(ud, &ct);
}

void usblogdevInit(struct usblogdev* ul);
void usblogdevDestroy(struct usblogdev* ul);

#endif
